import unittest

from tart.imaging.test.test_correlator import TestCorrelator
from tart.imaging.test.test_correlator import TestHilbert
from tart.imaging.test.test_antenna_model import TestAntennaModel
from tart.imaging.test.test_correlator import TestCorrelator
from tart.imaging.test.test_correlator import TestHilbert
from tart.imaging.test.test_ephemerides_proxy import TestEphemeridesProxy
from tart.imaging.test.test_ephemeris import TestEphemeris
from tart.imaging.test.test_gps_satellite import TestGpsSatellite
from tart.imaging.test.test_gps_time import TestGpsTime
from tart.imaging.test.test_location import TestLocation
from tart.imaging.test.test_sun import TestSun
from tart.imaging.test.test_tart_util import TestTartUtil  # get rid of this!!!!
from tart.imaging.test.test_visibility import TestVisibility

if __name__ == "__main__":
    unittest.main()
